local function IsDST()
	return TheSim:GetGameID() ~= "DS" 
end

local EdwardelricRainLeg = Class(function(self, inst)
	self.inst = inst
	self._onmoisturedelta = function(inst,data)
		if (data.new > 0 and data.old <= 0) and not self.inst:HasTag("playerghost") then 
			self:StartSufferFromRain()
		elseif (data.new <= 0 and data.old > 0) or self.inst:HasTag("playerghost") then 
			self:ResumeFromRain()
		end
	end
	inst:ListenForEvent(IsDST() and "moisturedelta" or "moisturechange",self._onmoisturedelta)
end)

function EdwardelricRainLeg:OnRemoveFromEntity()
	inst:RemoveEventCallback(IsDST() and "moisturedelta" or "moisturechange",self._onmoisturedelta)
end

function EdwardelricRainLeg:DoRainDamage()
	local moisture_rate = IsDST() and self.inst.components.moisture:GetRate() 
	or (self.inst.components.moisture:GetMoistureRate() + self.inst.components.moisture:GetEquippedMoistureRate())
	self.inst.components.health:DoDelta(moisture_rate >= 0 and -0.4 or -0.2,true, "water")
end

function EdwardelricRainLeg:StartSufferFromRain()
	if self.RainLegTask then 
		print(self.inst,"[EdwardelricRainLeg]:Warning,old RainLegTask duplicated")
		self.RainLegTask:Cancel()
		self.RainLegTask = nil 
	end 
	print(self.inst,"Start to suffer from rain")
	self.inst.components.talker:Say("太潮湿了，我的机械臂腿在嘎吱作响。要是温蒂在就好了。")
	if IsDST() then 
		self.inst.components.locomotor:SetExternalSpeedMultiplier(self.inst,"EdwardelricRainLeg",0.85)
	else
		self.inst.components.locomotor:AddSpeedModifier_Mult("EdwardelricRainLeg", -0.15)
	end
	self.RainLegTask = self.inst:DoPeriodicTask(1,function()
		self:DoRainDamage()
	end)
end

function EdwardelricRainLeg:ResumeFromRain()
	print(self.inst,"Resume from rain")
	if self.RainLegTask then 
		self.RainLegTask:Cancel()
		self.RainLegTask = nil 
	end 
	self.inst.components.talker:Say("总算干了，希望别留下什么问题。温蒂会杀了我的！")
	if IsDST() then 
		self.inst.components.locomotor:RemoveExternalSpeedMultiplier(self.inst,"EdwardelricRainLeg")
	else
		self.inst.components.locomotor:RemoveSpeedModifier_Mult("EdwardelricRainLeg")
	end 
end

return EdwardelricRainLeg